// ButtonsAndCountersFrame.java
//
// Informatics 45 Spring 2010
// Code Example: JPanels, Nested Layout, Model Separation, and Custom Events
// Version 1
//
// This version of the "buttons and counters" frame demonstrates JFrames and
// nested layout.  The goal is to lay out the buttons and counters differently
// than in the previous code example (Layout and Event Handling in Swing), so
// that the counter for each button appears directly below the button, rather
// than having all the buttons and controls laid out horizontally.
//
// We're now in the position where we're trying to build a two-dimensional
// layout: a row of button/label combinations, where each label appears below
// a button.  A single BoxLayout is incapable of generating this kind of
// layout; BoxLayout only knows how to lay out a sequence of controls in one
// dimension (either horizontally or vertically), by giving each control as
// much space as it asks for and placing them outside of the viewable area of
// the window if they, collectively, ask for too much space.
//
// Just as we can break complex classes or methods into smaller ones, we can
// also take complex layouts and break them into smaller ones.  What we need
// are containers that can hold multiple controls (buttons, labels, list boxes,
// etc.) and lay them out internally, then to collect up the containers into
// our main layout.  For example, in the case of our two-dimensional buttons
// and counters example, where each counter lies below its button, we can
// create four containers, each holding a button and its associated label.
// The four containers could each use a vertical BoxLayout to lay out their
// components, while the frame's content pane could use a horizontal BoxLayout
// to lay out their components next to each other left-to-right.
//
// The Java class that implements these containers is JPanel: a JPanel is a
// container that can hold other controls, complete with its own internal
// layout manager.
//
// This version of the code example implements everything in the frame class:
//
// * Three integer variables to hold the counters for "Hello", "Goodbye",
//   and "Boo".
// * The code that creates the various controls and layouts, including
//   JPanels nested within the main layout.
// * The action listeners that handle the button presses.
// * The code that increments the counters.
// * The code that determines when it's time to redraw the frame with
//   updated values.

package inf45.spring2010.examples.gui2;

// We often need four imports for GUI work in Swing: java.awt.*,
// java.awt.event.*, javax.swing.*, and javax.swing.event.*.  I
// usually put all four of those in place when I start a new GUI
// class, then remove the ones I don't need (because otherwise I'll
// be left with warnings that I'll have a hard time distinguishing
// from warnings that matter).

import java.awt.event.*;
import javax.swing.*;


public class ButtonsAndCountersFrame extends JFrame
{
	// These three variables contain the three counters that keep track
	// of how many times the "Hello", "Goodbye", and "Boo" buttons have
	// been pressed.
	private int helloCount;
	private int goodbyeCount;
	private int booCount;

	// These three labels contain the text displayed beneath the "Hello",
	// "Goodbye", and "Boo" buttons.  The reason these are stored as
	// fields is that we'll need to access them in multiple places within
	// the class.  Notice that we haven't made any of the other controls
	// into fields of the class, since we never need to change any of the
	// others once they're built.
	private JLabel helloLabel;
	private JLabel goodbyeLabel;
	private JLabel booLabel;
	
	
	public ButtonsAndCountersFrame()
	{
		// We'll set up some properties of the frame: the title to be
		// displayed in the title bar, the initial size of the window,
		// and the action to take when the window is closed -- in this
		// case, we want to dispose of it, which will have the effect
		// of ending the program, since this is its only window.
		setTitle("Informatics 45 Spring 2009 - GUI Example #2");
		setSize(500, 150);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		// We'll also initialize all the counters to zero.
		helloCount = 0;
		goodbyeCount = 0;
		booCount = 0;
		
		// Finally, we'll build our user interface.
		buildUI();
	}
	
	
	private void buildUI()
	{
		// The main layout for our frame's content pane is a horizontal (X_AXIS)
		// BoxLayout, consisting of the four "button panels" (the "hello panel",
		// "goodbye panel", and so on), separated by horizontal struts to
		// provide empty space between them, and surrounded by horizontal glue
		// to use up the remaining empty space in the window.
		
		BoxLayout mainLayout = new BoxLayout(getContentPane(), BoxLayout.X_AXIS);
		getContentPane().setLayout(mainLayout);

		getContentPane().add(Box.createHorizontalGlue());
		getContentPane().add(buildHelloPanel());
		
		getContentPane().add(Box.createHorizontalStrut(10));
		getContentPane().add(buildGoodbyePanel());
		
		getContentPane().add(Box.createHorizontalStrut(10));
		getContentPane().add(buildBooPanel());
		
		getContentPane().add(Box.createHorizontalStrut(10));
		getContentPane().add(buildEndPanel());
		
		getContentPane().add(Box.createHorizontalGlue());

		// We need to do this to make sure that the count labels will be
		// initialized to the correct value right from the start, rather than
		// only having a value after the first button is pressed.
		updateCountLabels();
	}

	
	// The next four methods build the four button panels, which combine each
	// button with its corresponding label.  We'll build these as JPanels and
	// return them.  The JPanels will use a vertical (Y_AXIS) BoxLayout, so
	// that the button will appear above the label.  We'll also wire up each
	// button to an action listener, as we did in the previous code example;
	// each button has its own method that is called when its action listener
	// is called.
	//
	// It should bother you to write four methods like these, which are largely
	// identical to one another.  In class, when we had finished writing the
	// second one and determined the level of similarity between them, we
	// stopped and combined them.  The next version of this example explores
	// this change.
	
	private JPanel buildHelloPanel()
	{
		JPanel helloPanel = new JPanel();
		BoxLayout helloLayout = new BoxLayout(helloPanel, BoxLayout.Y_AXIS);
		helloPanel.setLayout(helloLayout);
		
		getContentPane().add(helloPanel);
		
		JButton helloButton = new JButton("Hello");
		helloButton.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					helloPressed();
				}
			});
		helloPanel.add(helloButton);

		helloPanel.add(Box.createVerticalStrut(5));
		
		helloLabel = new JLabel();
		helloPanel.add(helloLabel);
		
		return helloPanel;
	}
	
	
	private JPanel buildGoodbyePanel()
	{
		JPanel goodbyePanel = new JPanel();
		BoxLayout goodbyeLayout = new BoxLayout(goodbyePanel, BoxLayout.Y_AXIS);
		goodbyePanel.setLayout(goodbyeLayout);
		
		getContentPane().add(goodbyePanel);
		
		JButton goodbyeButton = new JButton("Goodbye");
		goodbyeButton.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					goodbyePressed();
				}
			});
		goodbyePanel.add(goodbyeButton);

		goodbyePanel.add(Box.createVerticalStrut(5));
		
		goodbyeLabel = new JLabel();
		goodbyePanel.add(goodbyeLabel);
		
		return goodbyePanel;
	}
	
	
	private JPanel buildBooPanel()
	{
		JPanel booPanel = new JPanel();
		BoxLayout booLayout = new BoxLayout(booPanel, BoxLayout.Y_AXIS);
		booPanel.setLayout(booLayout);
		
		getContentPane().add(booPanel);
		
		JButton booButton = new JButton("Boo");
		booButton.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					booPressed();
				}
			});
		booPanel.add(booButton);

		booPanel.add(Box.createVerticalStrut(5));
		
		booLabel = new JLabel();
		booPanel.add(booLabel);

		return booPanel;
	}
	
	
	private JPanel buildEndPanel()
	{
		JPanel endPanel = new JPanel();
		BoxLayout endLayout = new BoxLayout(endPanel, BoxLayout.Y_AXIS);
		endPanel.setLayout(endLayout);
		
		getContentPane().add(endPanel);
		
		JButton endButton = new JButton("End");
		endButton.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					endPressed();
				}
			});
		endPanel.add(endButton);

		endPanel.add(Box.createVerticalStrut(5));
		
		JLabel endLabel = new JLabel("!!!");
		endPanel.add(endLabel);
		
		return endPanel;
	}

	
	// These are the four methods that contain the code to be executed when
	// each button is pressed.  It's necessary for each of the first three to
	// call updateCountLabels() after incrementing one of the counters.  The
	// reason is that Java doesn't bind data, like an int variable, to the
	// GUI control that displays its value automatically; it becomes our 
	// problem to tell the GUI to update its labels, so they can be redrawn
	// when they need to be.  In our case, we have one method called
	// updateCountLabels() that does this updating, then we make sure to
	// call that method whenever one of the labels is incremented.

	private void helloPressed()
	{
		helloCount++;
		updateCountLabels();
	}
	
	
	private void goodbyePressed()
	{
		goodbyeCount++;
		updateCountLabels();
	}
	
	
	private void booPressed()
	{
		booCount++;
		updateCountLabels();
	}
	
	
	private void endPressed()
	{
		dispose();
	}


	private void updateCountLabels()
	{
		helloLabel.setText(buildCountText(helloCount));
		goodbyeLabel.setText(buildCountText(goodbyeCount));
		booLabel.setText(buildCountText(booCount));
	}
	
	
	private String buildCountText(int count)
	{
		return "Count: " + count;
	}
}
