// ButtonsAndCountersFrame.java
//
// Informatics 45 Spring 2010
// Code Example: JPanels, Nested Layout, Model Separation, and Custom Events
// Version 3
//
// One of the things that begins to happen as your programs get to be
// progressively larger is that it becomes increasingly difficult to keep
// all the details of your program in your head simultaneously.  The solution
// to this problem is to try to separate your program into littler pieces --
// which, ideally, depend on as few details of the other as possible -- so
// that you can work on one part of your program without having to bear in
// mind all the details of the other parts, focusing only on how the part
// you're working interacts directly with others.  There is a fancy name for
// this: "separation of concerns."
//
// In a program with a graphical user interface, there is generally a natural
// separation between the code that performs the underlying "business logic"
// -- in the case of this example, the code that stores and updates the
// counters -- and the code that provides the user interface.  We often refer
// to the underlying data and logic as the "model" and the user interface as
// the "view."  (There is a more complex pattern that introduces a third
// component, the "controller," which arbitrates the relationship between the
// model and the view; for the sake of clarity, I'll leave that out of this
// example.)
//
// This version of our code example houses our business logic in a separate
// class called Counters, which holds the values of the three integers and
// exposes methods to get their values and to increment them.  (Note that we
// don't expose a method to set the value of a counter; instead, we only provide
// a method to increment one.  The reason is that we want to view to be as
// "dumb" as possible, leaving as much of the actual work to the model as
// possible.  Better for the model to be the one to decide how to increment a
// counter, rather than having to view get the value of a counter, add one to
// it, then set the new value.  After all, as simple as it sounds, it's still
// part of our business logic, the underlying "smarts" of our program.
// 
// Beyond that, the program hasn't changed much.  We replaced the three integer
// counters in this class with a Counters object, then replaced the direct use
// of the three integers with calls to Counters methods.

package inf45.spring2010.examples.gui2;

import java.awt.event.*;
import javax.swing.*;


public class ButtonsAndCountersFrame extends JFrame
{
	private Counters counters;
	
	private JLabel helloLabel;
	private JLabel goodbyeLabel;
	private JLabel booLabel;
	
	
	public ButtonsAndCountersFrame()
	{
		setTitle("Informatics 45 Spring 2009 - GUI Example #2");
		setSize(500, 150);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		counters = new Counters();
		
		buildUI();
	}
	
	
	private void buildUI()
	{
		BoxLayout mainLayout = new BoxLayout(getContentPane(), BoxLayout.X_AXIS);
		getContentPane().setLayout(mainLayout);

		getContentPane().add(Box.createHorizontalGlue());
		helloLabel = new JLabel();
		ActionListener helloActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					helloPressed();
				}
			};
		buildButtonPanel("Hello", helloLabel, helloActionListener);

		getContentPane().add(Box.createHorizontalStrut(10));
		goodbyeLabel = new JLabel();
		ActionListener goodbyeActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					goodbyePressed();
				}
			};
		buildButtonPanel("Goodbye", goodbyeLabel, goodbyeActionListener);

		getContentPane().add(Box.createHorizontalStrut(10));
		booLabel = new JLabel();
		ActionListener booActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					booPressed();
				}
			};
		buildButtonPanel("Boo", booLabel, booActionListener);
		
		getContentPane().add(Box.createHorizontalStrut(10));
		JLabel endLabel = new JLabel("!!!");
		ActionListener endActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					endPressed();
				}
			};
		buildButtonPanel("End", endLabel, endActionListener);
		
		getContentPane().add(Box.createHorizontalGlue());
		
		updateCountLabels();
	}
	
	
	private void buildButtonPanel(
		String buttonText, JLabel label, ActionListener buttonActionListener)
	{
		JPanel buttonPanel = new JPanel();
		BoxLayout buttonPanelLayout = new BoxLayout(buttonPanel, BoxLayout.Y_AXIS);
		buttonPanel.setLayout(buttonPanelLayout);
		
		getContentPane().add(buttonPanel);
		
		JButton button = new JButton(buttonText);
		button.addActionListener(buttonActionListener);
		buttonPanel.add(button);

		buttonPanel.add(Box.createVerticalStrut(5));
		
		buttonPanel.add(label);
	}

	
	// Here, we increment the counters by calling methods on the Counters
	// object, rather than directly incrementing the counters (as we were
	// doing before).

	private void helloPressed()
	{
		counters.incrementHelloCount();
		updateCountLabels();
	}
	
	
	private void goodbyePressed()
	{
		counters.incrementGoodbyeCount();
		updateCountLabels();
	}
	
	
	private void booPressed()
	{
		counters.incrementBooCount();
		updateCountLabels();
	}
	
	
	private void endPressed()
	{
		dispose();
	}

	
	// Here, we update the labels by fetching the counter values from the
	// Counters object, rather than from three integer variables within the
	// frame (as we were doing before).

	private void updateCountLabels()
	{
		helloLabel.setText(buildCountText(counters.getHelloCount()));
		goodbyeLabel.setText(buildCountText(counters.getGoodbyeCount()));
		booLabel.setText(buildCountText(counters.getBooCount()));
	}
	
	
	private String buildCountText(int count)
	{
		return "Count: " + count;
	}
}
