// ButtonsAndCountersFrame.java
//
// Informatics 45 Spring 2010
// Code Example: JPanels, Nested Layout, Model Separation, and Custom Events
// Version 4
//
// This version of the frame does not decide on its own when the text in
// the labels should be updated.  Instead, it relies on the view to tell
// it when the counters have changed, by using a custom listener called a
// "counters changed" listener -- which is very similar, in concept, to the
// action listeners that get attached to JButtons.
//
// The change seems small, but it's quite profound in general.  While it's
// relatively obvious when the counter labels should be updated in this
// user interface, it's not always clear when user interface elements need
// to be updated with new values.  The less we can tell our views about this,
// the more easily we can change our business logic -- the "guts" of our
// program -- without affecting our view.  So if we rely on the model to tell
// the view when change has occurred, the view can become blissfully unaware
// of what causes change to occur in the first place.  The view, then, knows
// less about the model (the underlying business logic) and the model, in turn,
// knows less about the view because it deals only with abstract "counters
// changed listeners" without knowing who they are or why they're listening.

package inf45.spring2010.examples.gui2;

import java.awt.event.*;
import javax.swing.*;


public class ButtonsAndCountersFrame extends JFrame
{
	private Counters counters;
	
	private JLabel helloLabel;
	private JLabel goodbyeLabel;
	private JLabel booLabel;
	
	
	public ButtonsAndCountersFrame()
	{
		setTitle("Informatics 45 Spring 2009 - GUI Example #2");
		setSize(500, 150);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		// We wire up our view as a "counters changed" listener just as we
		// wired up action listeners to each of our buttons.  What this means
		// is "Whenever the Counters say they've changed, update the count
		// labels in the user interface."
		counters = new Counters();
		counters.addCountersChangedListener(
			new CountersChangedListener()
			{
				public void countersChanged()
				{
					updateCountLabels();
				}
			});
		
		buildUI();
		
		// We do have to call this method once as we're building our UI, so
		// that the labels will have the correct text initially.  After that,
		// the model will tell the view when updates need to occur.
		updateCountLabels();
	}
	
	
	private void buildUI()
	{
		BoxLayout mainLayout = new BoxLayout(getContentPane(), BoxLayout.X_AXIS);
		getContentPane().setLayout(mainLayout);

		getContentPane().add(Box.createHorizontalGlue());
		helloLabel = new JLabel();
		ActionListener helloActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					helloPressed();
				}
			};
		buildButtonPanel("Hello", helloLabel, helloActionListener);

		getContentPane().add(Box.createHorizontalStrut(10));
		goodbyeLabel = new JLabel();
		ActionListener goodbyeActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					goodbyePressed();
				}
			};
		buildButtonPanel("Goodbye", goodbyeLabel, goodbyeActionListener);

		getContentPane().add(Box.createHorizontalStrut(10));
		booLabel = new JLabel();
		ActionListener booActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					booPressed();
				}
			};
		buildButtonPanel("Boo", booLabel, booActionListener);
		
		getContentPane().add(Box.createHorizontalStrut(10));
		JLabel endLabel = new JLabel("!!!");
		ActionListener endActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					endPressed();
				}
			};
		buildButtonPanel("End", endLabel, endActionListener);
		
		getContentPane().add(Box.createHorizontalGlue());
	}
	
	
	private void buildButtonPanel(
		String buttonText, JLabel label, ActionListener buttonActionListener)
	{
		JPanel buttonPanel = new JPanel();
		BoxLayout buttonPanelLayout = new BoxLayout(buttonPanel, BoxLayout.Y_AXIS);
		buttonPanel.setLayout(buttonPanelLayout);
		
		getContentPane().add(buttonPanel);
		
		JButton button = new JButton(buttonText);
		button.addActionListener(buttonActionListener);
		buttonPanel.add(button);

		buttonPanel.add(Box.createVerticalStrut(5));
		
		buttonPanel.add(label);
	}


	private void helloPressed()
	{
		counters.incrementHelloCount();
	}
	
	
	private void goodbyePressed()
	{
		counters.incrementGoodbyeCount();
	}
	
	
	private void booPressed()
	{
		counters.incrementBooCount();
	}
	
	
	private void endPressed()
	{
		dispose();
	}


	public void updateCountLabels()
	{
		helloLabel.setText(buildCountText(counters.getHelloCount()));
		goodbyeLabel.setText(buildCountText(counters.getGoodbyeCount()));
		booLabel.setText(buildCountText(counters.getBooCount()));
	}
	
	
	private String buildCountText(int count)
	{
		return "Count: " + count;
	}
}
