// ButtonsAndCountersFrame.java
//
// Informatics 45 Spring 2010
// Code Example: JPanels, Nested Layout, Model Separation, and Custom Events
// Version 5
//
// This version of the frame replaces the single Counters object with
// three Counter objects instead.  This allows the view to separately
// wire up event handlers for each counter's change, so that only one
// label will have to be updated at a time.  This is a performance win,
// since the user interface doesn't end up spending time updating data
// that hasn't changed; each time a button is pressed, only the corresponding
// label will be updated.
//
// The tradeoff is that the frame needs to hold three Counter objects
// instead of one Counters object.  On the other hand, it's simpler to
// write one Counter class than write a Counters class that maintains
// three separate counters (which requires us to duplicate code that
// got and incremented each of the counters).
//
// NOTE: The next stage in the evolution of this example would be to
// generalize the model and the view so that neither would be hard-coded
// to have certain button names, but instead to allow this to be controlled
// by, say, an ArrayList<String> passed to the frame's constructor.  This
// way you could have any combination of buttons while changing only one
// the main() method.  In fact, you could read the Strings from a file,
// allowing the program to be totally customizable without ever changing
// any code at all.  I haven't provided this version of the example, but you
// should challenge yourself to put it together as an exploration if you have
// the time.

package inf45.spring2010.examples.gui2;

import java.awt.event.*;
import javax.swing.*;


public class ButtonsAndCountersFrame extends JFrame
{
	private Counter helloCounter;
	private Counter goodbyeCounter;
	private Counter booCounter;
	
	private JLabel helloLabel;
	private JLabel goodbyeLabel;
	private JLabel booLabel;
	
	
	public ButtonsAndCountersFrame()
	{
		setTitle("Informatics 45 Spring 2009 - GUI Example #2");
		setSize(500, 150);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		// Here, we wire up a separate "counter changed" listener to each of
		// the counters as we create them.  Note that the listener can be
		// configured to update the appropriate counter, rather than having
		// only a single method to update all of them.

		helloCounter = new Counter();
		helloCounter.addCounterChangedListener(
			new CounterChangedListener()
			{
				public void counterChanged()
				{
					updateCountLabel(helloLabel, helloCounter);
				}
			});
		
		goodbyeCounter = new Counter();
		goodbyeCounter.addCounterChangedListener(
			new CounterChangedListener()
			{
				public void counterChanged()
				{
					updateCountLabel(goodbyeLabel, goodbyeCounter);
				}
			});
		
		booCounter = new Counter();
		booCounter.addCounterChangedListener(
			new CounterChangedListener()
			{
				public void counterChanged()
				{
					updateCountLabel(booLabel, booCounter);
				}
			});
		
		buildUI();
		
		// We do have to call these methods once as we're building our UI, so
		// that the labels will have the correct text initially.  After that,
		// the model will tell the view when updates need to occur.
		updateCountLabel(helloLabel, helloCounter);
		updateCountLabel(goodbyeLabel, goodbyeCounter);
		updateCountLabel(booLabel, booCounter);
	}
	
	
	private void buildUI()
	{
		BoxLayout mainLayout = new BoxLayout(getContentPane(), BoxLayout.X_AXIS);
		getContentPane().setLayout(mainLayout);

		getContentPane().add(Box.createHorizontalGlue());
		helloLabel = new JLabel();
		ActionListener helloActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					helloPressed();
				}
			};
		buildButtonPanel("Hello", helloLabel, helloActionListener);

		getContentPane().add(Box.createHorizontalStrut(10));
		goodbyeLabel = new JLabel();
		ActionListener goodbyeActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					goodbyePressed();
				}
			};
		buildButtonPanel("Goodbye", goodbyeLabel, goodbyeActionListener);

		getContentPane().add(Box.createHorizontalStrut(10));
		booLabel = new JLabel();
		ActionListener booActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					booPressed();
				}
			};
		buildButtonPanel("Boo", booLabel, booActionListener);
		
		getContentPane().add(Box.createHorizontalStrut(10));
		JLabel endLabel = new JLabel("!!!");
		ActionListener endActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					endPressed();
				}
			};
		buildButtonPanel("End", endLabel, endActionListener);
		
		getContentPane().add(Box.createHorizontalGlue());
	}
	
	
	private void buildButtonPanel(
		String buttonText, JLabel label, ActionListener buttonActionListener)
	{
		JPanel buttonPanel = new JPanel();
		BoxLayout buttonPanelLayout = new BoxLayout(buttonPanel, BoxLayout.Y_AXIS);
		buttonPanel.setLayout(buttonPanelLayout);
		
		getContentPane().add(buttonPanel);
		
		JButton button = new JButton(buttonText);
		button.addActionListener(buttonActionListener);
		buttonPanel.add(button);

		buttonPanel.add(Box.createVerticalStrut(5));
		
		buttonPanel.add(label);
	}


	private void helloPressed()
	{
		helloCounter.incrementCount();
	}
	
	
	private void goodbyePressed()
	{
		goodbyeCounter.incrementCount();
	}
	
	
	private void booPressed()
	{
		booCounter.incrementCount();
	}
	
	
	private void endPressed()
	{
		dispose();
	}


	public void updateCountLabel(JLabel label, Counter counter)
	{
		label.setText(buildCountText(counter.getCount()));
	}
	
	
	private String buildCountText(int count)
	{
		return "Count: " + count;
	}
}
